/** Selector function.
 *    Args:
 *      id: a string that contains  #id,  .class  or  element
 *    Returns:
 *      HTMLElement or Array of HTMLElements
 */
function $(pattern, scope){
  if(!scope){var scope = document};
  var selectors = pattern.split(/\s*?,\s*?/);
  var results = [];
  for(var s=0; s<selectors.length; s++){ selector = selectors[s];
    if(selector.charAt(0)=="#"){
      var e = scope.getElementById(selector.slice(1));
      if(selectors.length==1){return e} else{results.push(e)};
    } else if(selector.charAt(0)=="."){
      if(document.getElementsByClassName){ results = results.concat( Array.prototype.slice.call(scope.getElementsByClassName(selector.slice(1))) )
      } else{ var cs = scope.getElementsByTagName("*"); if(cs.length==0){continue};
      var regexp = new RegExp('\\b'+selector.slice(1)+'\\b', 'gi');
      for(var i=0;i<cs.length;i++){ if(cs[i].className.search(regexp)!=-1){results.push(cs[i])} };
      }
    } else if(selector.charAt(0)=="["&&selector.slice(-1)=="]"){
      var attribute = selector.match(/[^\[\]\=]+/)[0];
      var value = selector.match(/\=[^\]]+(?=\])/); if(value!=null) value = value[0].slice(1);
      var cs = scope.getElementsByTagName("*"); if(cs.length==0){continue};
      for(var i=0;i<cs.length;i++){
        var val = cs[i].getAttribute(attribute);
        if(val!=null&&((value!=null)?(val==value):true)){results.push(cs[i])}
      };
    } else{
      var cs = scope.getElementsByTagName(selector);
      for(var i=0;i<cs.length;i++){results.push(cs[i])};
    }
  }
  return results;
}


function callback(name, data){
  // if(!/SketchUp/i.test(navigator.userAgent)) return; Only SU8Win has "SketchUp" in the user agent!
  window.location.href = "skp:" + name + "@" + data;
}



/** Check if an object contains a certain value in a key/value pair.
 *    Args:
 *      value: a string/number/boolean
 *    Returns:
 *      true/false
 */
function inObject(value, object){for(i in object){ if(i==value){return true} };return false}



function hasClass(elem, className){ return new RegExp("(^|\\b)"+className+"(\\b|$)").test(elem.className) }



function addClass(elem, className){if(!hasClass(elem,className)) elem.className += (elem.className ? ' ' : '')+className}



function removeClass(elem, className){ var r = new RegExp("(^\\s*|\\s*\\b)"+className+"(\\b|$)"); elem.className = elem.className.replace(r,"")}



// Show element.
function show(elem){
  elem.style.display = elem.orig_display || "block";
}



// Hide element.
function hide(elem){
  // remember original display property because it could be block,inline-block,inline etc.
  elem.orig_display = elem.display;
  elem.style.display = "none";
}



// Debug function.
function puts(s){callback('puts', JSON.stringify(s))}



//  Resize the height of the webdialog so that everything fits well.
//  Measures the HTML document's width and height and send them to Ruby.
function resizeHeight() {
  if(isResizing==false){
    callback('resizeHeight', '['+(document.body.clientWidth||document.body.scrollWidth)+','+(document.body.clientHeight||document.body.scrollHeight)+']');
    window.setTimeout(function(){isResizing=false}, 500);
  }
  isResizing = true;
}
var isResizing = false;



/** Tabs with tabs bar.
 *    Args:
 *      elem: a container containing one .tabs-nav and several fieldset.tabs-panel with name attribute or legend
 *      def: the name of the tab to be preselected
 */
function Tabs(elem, def){
  elem.tabsset = this;
  var _this = this;
  var nav = $(".tabs-nav",elem)[0];
  this.tabs = []; // list of tab numbers
  this.names = {};
  this.panels = {};  // hash of nr => panel element
  this.buttons = {}; // hash of nr => button element
  var counter = 0;
  this.onAdded = function(){};
  this.onRemoved = function(){};
  // functions
  this.add = function(name, panel){
    var nr = counter;
    counter++;
    this.tabs.push(nr);
    this.names[nr] = name;
    this.panels[nr] = panel;
    var b = document.createElement("li");
    b.setAttribute("class","tabs-button");
    b.setAttribute("title",name);
    var n = document.createElement("span");
    n.innerHTML = name;
    b.appendChild(n);
    this.buttons[nr] = b;
    nav.appendChild(b);
    var c = document.createElement("span");
    c.setAttribute("class","tabs-button-close");
    c.setAttribute("title","remove");
    c.innerHTML = "×";
    b.appendChild(c);
    b.nr = nr;
    panel.nr = nr;
    hide(panel);
    b.onclick = function(){ _this.select(nr) };
    c.onclick = function(event){
      if(!event){var event = window.event};
      if(event.stopPropagation){event.stopPropagation()};
      event.cancelBubble = true;
      event.returnValue = false;
      _this.remove(nr)
    };
    this.onAdded(nr);
    return nr;
  };
  this.duplicate = function(newname, template){
    var newpanel = template.cloneNode(true);
    elem.insertBefore(newpanel, template);
    this.add(newname, newpanel);
    this.select(newpanel.nr);
    return newpanel.nr;
  }
  this.select = function(nr){
    for(var i=0; i<this.tabs.length; i++){removeClass(this.buttons[this.tabs[i]], "active")};
    addClass(this.buttons[nr], "active");
    for(var i=0; i<this.tabs.length; i++){hide(this.panels[this.tabs[i]])};
    show(this.panels[nr]);
    resizeHeight();
  };
  this.changeName = function(nr, newname){
    this.buttons[nr].setAttribute("title",newname);
    this.buttons[nr].getElementsByTagName("span")[0].innerHTML = newname;
    this.names[nr] = newname;
  };
  this.remove = function(nr){
    var b = this.buttons[nr];
    nav.removeChild(b);
    delete this.buttons[nr];
    var p = this.panels[nr];
    elem.removeChild(p);
    delete this.panels[nr];
    delete this.names[nr];
    for(var i=0; i<this.tabs.length; i++){ if(this.tabs[i]==nr){ this.tabs.splice(i,1) } };
    if(this.tabs.length>0){ this.select(this.tabs[i-1]||this.tabs[0]) };
    this.onRemoved(nr);
    return nr;
  };
  // create buttons
  var panels = $(".tabs-panel",elem); // tmp
  for(var i=0; i<panels.length; i++){
    var panel = panels[i]; var name = (panel.name)? panel.name : (($("legend", panel).length!=0)? $("legend", panel)[0].innerHTML : i );
    this.add(name, panel)
  };
  // set default
  if(def){
    this.select(def);
  }else if(this.tabs.length>0){
    this.select(this.tabs[0]);
  }
}



/** Disable or enable input elements in cascaded forms.
 *    Args:
 *      input:
 *      1) A checkbox or radio button that has an rel attribute pointing at
 *         the id of another input/container that should be disabled/enabled.
 *         <somewhere> <checkbox rel="c1"> </somewhere> ...
 *         <somewhereelse> <container id="c1"> <input> </container> </somewhereelse>
 *      2) A container that has only one checkbox/radio button as direct child element.
 *         All other inputs cascaded in subcontainers of this container will be disabled/enabled.
 *         <container> <checkbox> <subcontainer1> <input> </subcontainer1> </container>
 *     Returns:
 *       nothing
 */
function toggle_disabled(input){
  var toggle_children = function(input){
    // rel attribute
    if(input.getAttribute('rel')){
      var rel_container = $("#"+input.getAttribute('rel'));
      rel_container.disabled = !input.checked;
      var rel_inputs = $('input',rel_container);
      for(var i=0; i<rel_inputs.length; i++){
        rel_inputs[i].disabled = !input.checked;
      }
    // parent has only one child input and several sub-containers
    } else{
      c=input.parentNode.childNodes
      if(input.parentNode.nodeName=="LABEL"){c=input.parentNode.parentNode.childNodes}
      n=[]; for(var i=0;i<c.length;i++){
        if(c[i].nodeName=="INPUT"&&(c[i].type=="checkbox"||c[i].type=="radio")) n.push(c[i])
        else if(c[i].nodeName=="LABEL"){
          d=c[i].childNodes;
          for(var j=0;j<d.length;j++){if(d[j].nodeName=="INPUT"&&(d[j].type=="checkbox"||d[j].type=="radio")) n.push(d[j])}
        }
      }
      if(n.length==1){
        for(var i=0; i<c.length; i++){ if(c[i].childNodes.length==0){continue}
          ci=$('input', c[i]);
          for(var k=0; k<ci.length; k++){if(!(ci[k]==input)) ci[k].disabled = !input.checked;}
        }
      }
    }
  }
  toggle_children(input);
  // radio: loop over all connected radio buttons (with same name)
  if(input.type=='radio'){
    radios = $("[name="+input.name+"]");
    for(var i=0; i<radios.length; i++){ if(radios[i]!=input){ toggle_children(radios[i]) } }
  }
}




/** Init and fill all default values into the matching input elements
 *    Args:
 *      options: object containing key/values where the key matches the name
 *        attribute of the corresponding input element
 *        {option_name: option_value}
 *    Returns:
 *      nothing
 */
function start(opt){
  if(!opt) opt = {}
  // measures the window's inner dimensions
  callback("windowInner", '['+(window.innerWidth||document.documentElement.clientWidth)+','+(window.innerHeight||document.documentElement.clientHeight)+']');
  // Create tabs
  var tabsset = new Tabs( $("#tabs") );
  tabsset.onAdded = function(nr){
    var panel = tabsset.panels[nr];
    $("[name=name]", panel)[0].value = tabsset.names[nr];
    $("[name=name]", panel)[0].onblur = function(){if(this.value!=""){tabsset.changeName(nr, this.value)}};
  };
  // create newtab button
  var template = tabsset.panels[0];
  hide(template);
  tabsset.changeName(0, "+");
  addClass(tabsset.buttons[0], "right");
  tabsset.buttons[0].removeChild( $(".tabs-button-close", tabsset.buttons[0])[0] );
  tabsset.buttons[0].onclick = function(){ tabsset.duplicate("new", template) };
  tabsset.tabs.shift();
  //
  tabsset.onRemoved = function(nr){
    if(tabsset.tabs.length==0){ tabsset.duplicate("new", template) };
  };
  //
  var optlength = 0;
  for(var id in opt){
    optlength++;
    var options = opt[id];
    var panel = null;
    var tabexists = false; for(var nr in tabsset.panels){if(tabsset.names[nr]==options["name"]){tabexists = true; panel = tabsset.panels[nr]}}
    if( !tabexists ){ var panel = tabsset.panels[tabsset.duplicate(options["name"], template)] }
    // load defaults
    // identifies input elements when their name matches the default option's name
    var inputs = $("input", panel);
    inputs = inputs.concat($("select"), panel);
    for(var j=0;j<inputs.length;j++){var i=inputs[j];
      if( inObject(i.name,options) && options[i.name]!="undefined" ){
        if(i.type=="checkbox"){ i.checked = options[i.name]; if(i.checked) addClass(i, "checked");
        } else if(i.type=="radio" && options[i.name]==i.value){ i.checked = true; addClass(i, "checked");
        } else if(i.type!="radio"){ i.value = options[i.name] }
      }
    };
    $("img", panel)[0].src = $("[name=icon]", panel)[0].value;
  };
  if(optlength < 2){tabsset.duplicate("new", template)}
  tabsset.select(tabsset.tabs[0]);
  resizeHeight();
  void(0);
}

/** Return all user input to Ruby
 *    Checkboxes are identified by their "name" attribute and return true/false.
 *    Other elements are identified by their "name" and return the "value" attribute.
 *    Args:
 *      nothing
 *    Returns:
 *      string: "{option_name=>option_value,...}"
 */
function returnOptions(){
  var user_input = [];
  var tabsset = $("#tabs").tabsset;
  for(var nr in tabsset.panels){
    var tab_input = [];
    var panel = tabsset.panels[nr];
    var inputs = $("input", panel);
    inputs = inputs.concat($("select", panel));
    for(var j=0;j<inputs.length;j++){i=inputs[j];
      if(i.name=="tool") continue;
      if(i.type=="checkbox"&&i.name&&!i.disabled){tab_input.push('"'+i.name+'" =\> '+i.checked)}
      else if(i.type=="radio"&&i.name&&!i.disabled&&i.checked){tab_input.push('"'+i.name+'" =\> "'+i.value+'"')}
      else if(i.type=="radio"&&i.name&&!i.disabled&&!i.checked){}
      else if(i.name&&!i.disabled&&!isNaN(i.value)&&(i.type=="text"&&hasClass(i,"num")||i.type=="number")){tab_input.push('"'+i.name+'" =\> '+parseInt(i.value))}
      else if(i.name&&!i.disabled){tab_input.push('"'+i.name+'" =\> '+JSON.stringify(i.value))}
    }
    user_input.push(JSON.stringify(nr) + " =\>  {" + tab_input.join(", ") + "}");
  }
  var user_input = "{"+user_input.join(", ")+"}";
  $("#results").value = user_input;
  callback('receiveOptions');
}
